#ifndef BIT_H
#define BIT_H

// Construct an integer with a single bit set at position 'bitpos'
#define BIT(bitpos) (1<<(bitpos))

// Set the bit located at position 'bit' in variable/register 'var'
#define bit_set(var, bit) ((var) |= BIT(bit))

// Clear the bit located at position 'bit' in variable/register 'var'
#define bit_clear(var, bit) ((var) &= ~BIT(bit))

// Toggle the bit located at position 'bit' in variable/register 'var'
#define bit_toggle(var, bit) ((var) ^= BIT(bit))

// Read the bit located at position 'bit' in variable/register 'var'
#define bit_read(var, bit) ((var) & BIT(bit))

// Set any bits in 'mask' in variable/register 'var'
#define bit_set_mask(var, mask) ((var) |= (mask))

// Clear any bits in 'mask' in variable/register 'var'
#define bit_clear_mask(var, mask) ((var) &= ~(mask))

// Toggle any bits in 'mask' in variable/register 'var'
#define bit_toggle_mask(var, mask) ((var) ^= (mask))

// Read any bits in 'mask' in variable/register 'var'
#define bit_read_mask(var, mask) ((var) & (mask))

#define BV  BIT
#define sbi bit_set
#define cbi bit_clear

#define SREG_I  7                               // global interrupt bit in SREG
#define CLI     (SREG &= 0x7F)                  // disable global interrupts
#define SEI     (SREG |= 0x80)                  // enable global interrupts

#endif          // BIT_H
