#include "config.h"
#include "crc.h"
#include "flash.h"
#include "adc12.h"
#include "heater.h"
#include <string.h>


// read config/cal data
int RdConfig(CfgStruct* cs) {
    if (RdFlash((char*)cs, SEGMENT_000, sizeof(CfgStruct))) {           // error reading config data
        return 1;
    }
    return calc_crc((char*)cs,sizeof(CfgStruct));                       // check for corrupt data
                                                                        // crc = 0 if ok
}


// save config/cal data
void WrConfig(CfgStruct* cs) {
    update_crc((char*)cs,sizeof(CfgStruct));                            // calc new CRC and append to packet
    WrFlash((char*)cs, SEGMENT_000, sizeof(CfgStruct));                 // write config data
}


// populate config/cal with default values
void DefaultCfg (CfgStruct* cfg) {
    int     i,j;

    strncpy(cfg->sn,DEF_SN,N_SN-1);
    cfg->adc_Vref = ADC12_VREF;
    cfg->Tcase.K[0] = DEF_T_CAL0;                          // thermistor curve fit, YSI44031
    cfg->Tcase.K[1] = DEF_T_CAL1;
    cfg->Tcase.K[2] = DEF_T_CAL2;
    cfg->Tcase.K[3] = DEF_T_CAL3;
    cfg->Tair.K[0] = DEF_T_CAL0;                           // thermistor curve fit, YSI44031
    cfg->Tair.K[1] = DEF_T_CAL1;
    cfg->Tair.K[2] = DEF_T_CAL2;
    cfg->Tair.K[3] = DEF_T_CAL3;
    cfg->Tcase.Setpt = DEF_HSP;                             // operating case temperature
    cfg->Tair.Setpt = 0.0;
    cfg->Tcase.Rref = DEF_RREF;                             // resistor used for half-bridge thermistor measurement
    cfg->Tair.Rref = DEF_RREF;
    cfg->Tstbl = DEF_TCASE_STBL;                            // default stabilized window
    cfg->Flow.K[0] = DEF_FLOW_CAL0;                         // default flow cals
    cfg->Flow.K[1] = DEF_FLOW_CAL1;
    cfg->Flow.K[2] = DEF_FLOW_CAL2;
    cfg->Flow.K[3] = DEF_FLOW_CAL3;
    cfg->outputInterval = DEF_OUTPUT_INT;                   // default output interval
    cfg->lpf[0] = DEF_LPF0;                                 // default low-pass filtering constants
    cfg->lpf[1] = DEF_LPF1;
    cfg->lpf[2] = DEF_LPF2;
    cfg->lpf[3] = DEF_LPF3;
    cfg->lpf[4] = DEF_LPF4;
    cfg->filter_id = 0;                                     // init filter ID & ch used
    cfg->filter_ch = 0;
    cfg->p = P_GAIN;                                        // set default PID settings
    cfg->i = I_GAIN;
    cfg->d = D_GAIN;
    cfg->led_mode = FIXED;                                  // set default LED mode and intensities
    cfg->red = DEF_RED_INT;
    cfg->grn = DEF_GRN_INT;
    cfg->blue = DEF_BLUE_INT;
    strncpy(cfg->sn,"00000\0",sizeof(cfg->sn));
}
