#ifndef _config_h_
#define _config_h_

#include "types.h"
#include "global.h"

#define DEF_RREF        20.0e3;         // default reference resistor for thermistor measurements

typedef struct {
    float   Rref,                       // reference resistance (half bridge)
            Setpt,                      // operating setpt (C)
            K[4];                       // poly cal constants
} Tinfo;


typedef struct {
    float   K[4];                       // poly cal constants
} FlowInfo;


typedef struct {                        // configuration data (saved in flash)
    char        sn[N_SN];               // serial number
    float       adc_Vref;               // MCU ADC reference voltage
    FlowInfo    Flow;                   // flow sensor cal info
    Tinfo       Tcase,                  // Tcase cal info
                Tair;                   // Taux cal info
    float       Tstbl;                  // case temperature stabilized window
    uint        outputInterval;         // number of 1-sec data to average for output stream
    float       lpf[N_LPF];             // low-pass filtering constants
    uint        filter_id;              // filter number for currently running filter
    uint        filter_ch;              // spot/channel of currently running filter
    float       p,i,d;                  // PID gain setings for case heater control
    uchar       led_mode;               // intensity mode (auto,fixed)
    float       red,grn,blue;           // LED settings
    uint        crc;                    // 16-bit crc for structure (last item in structure)
} CfgStruct;


int RdConfig (CfgStruct* cfg);          // read configuration
void WrConfig (CfgStruct* cfg);         // write configuration
void DefaultCfg (CfgStruct* cfg);       // populate config/cal with default values

#endif
