#ifndef __configuration_h__
#define __configuration_h__

#include "temperature.h"
#include "globals.h"

// CfgParamStruct flags
#define F_TXRF  1                   // enable transmitter RF: 0=no, 1=yes
#define F_PTU   2                   // enable PTU data out: 0=no, 1=yes
#define F_STAT  4                   // enable status data: 0=no, 1=yes
#define F_GPS   8                   // enable GPS data: 0=no, 1=yes
#define F_LEDS  16                  // enable LEDs: 0=off, 1=on
#define F_PTUX  32                  // enable enhanced PTU data: 0=off, 1=on

typedef struct {
    float P[5][5];                  // pressure
    float Ta[4];                    // ambient temperature
    float Tu[4];                    // humidity temperature
    ThermChar Tpr;                  // thermistor characterization for pressure temperature
    float U[8];                     // humidity
} CalCoeff;

typedef struct {
    char sn[13];                    // serial number
    char probe[13];                 // probe ID
    char sonde[13];                 // sonde ID
    int heater_sp;                  // heater set point (C)
    uchar flags;                    // operation flags
    float tx_freq[4];               // transmitter frequencies (MHz)
    CalCoeff cal_coeff;             // calibration coefficients
    uchar tx_mode;                  // transmission mode
    uchar rrPTU,rrGPS;              // report rates (0=disabled)
    uint Ucrc;                      // CRC for humidity look-up table in flash
    uint crc;                       // must be last in the structure (MSB first)
} CfgParamStruct;

void InitCfg (void);
uchar RdCfgFlash (void);
uchar WrCfgFlash (void);
uchar SwPos (void);
uchar GetSwPos (void);
float TxFreq (void);

#endif
