#include "global.h"
#include "heater.h"
#include "clock.h"

PIDstruct       pid[N_HEATERS];         // PID data structure for each heater

extern  volatile ulong  flags;          // from main.c

// init heater state and PID
void InitHeater (int htr, float Pg, float Ig, float Dg) {

    pid[htr].pGain = Pg;
    pid[htr].iGain = Ig;
    pid[htr].dGain = Dg;
    pid[htr].iState = 0.0;
    pid[htr].dState = 0.0;
}


// reset integral (wind-up)
void ResetI (int htr) {
    pid[htr].iState = 0.0;
}


// update PID values
float UpdatePID (int htr, float sv, float pv) {
    float       p,i,d,
                err;

    err = sv-pv;                                                    // calc error, + = turn heater on
    p = pid[htr].pGain * err;                                       // calc P term
    pid[htr].iState += err;                                         // calc I term
    if (pid[htr].iState > I_MAX) pid[htr].iState = I_MAX;           // clip I values (prevent wind-up)
    if (pid[htr].iState < I_MIN) pid[htr].iState = I_MIN;
    i = pid[htr].iState * pid[htr].iGain;                           // calc I output
    d = pid[htr].dGain * (pv - pid[htr].dState);                    // calc D term
    pid[htr].dState = pv;                                           // save for next D calc
    p = p + i - d;                                                  // summ outputs from P,I, & D
    if (p > 100.0) p = 100.0;                                       // full on
    if (p < 0.0) p = 0.0;                                           // off
    return p;
}


// update specified heater (n) on-time (%), 0-100%
bool SetHeater (int htr, float percent) __monitor {
    if ((percent >= 0)&&(percent <= 100)) {                     // validate
        switch (htr) {
            case CASE_HTR:                                      // set case heater
                TBCCR1 = (uint)((TB_INTERVAL-1) * (percent / 100.0));   // update PWM counter register
                break;
            case AUX_HTR:                                       // set aux heater
                TBCCR2 = (uint)((TB_INTERVAL-1) * (percent / 100.0));   // update PWM counter register
                break;
        }
        return true;
    } else {                                                    // error, must be between 0 and 100%
        return false;
    }
}
