#ifndef heater_h
#define heater_h

#include "types.h"

// default PID values
#define P_GAIN      500.0                   // proportinal gain
#define I_GAIN      50.0                    // integral gain
#define D_GAIN      50.0                    // derivative gain
#define I_MAX       2.0                     // I limits
#define I_MIN       -2.0
#define N_HEATERS   2

enum heaters {CASE_HTR, AUX_HTR};

typedef struct {
    float   pGain,
            iGain, iState,
            dGain, dState;
} PIDstruct;

void ResetI (int htr);                                                  // reset integral (wind-up)
void InitHeater (int htr, float Pg, float Ig, float Dg);                // prepare heater & PID

float UpdatePID (int htr, float sv, float pv);                          // update PID data for specified heater (htr),
                                                                        // providing sv = set value, pv = process value
                                                                        // return heater setting (0-100%)
                                                                        // call on regular interval (delta-time is constant)

bool SetHeater (int htr, float percent);                                // update specified heater (htr), on-time (0-100%)


#endif
