#include "led.h"
#include "clock.h"
#include "global.h"

extern volatile uint    sampleCnt;                  // from main.c
extern volatile ulong   flags;                      // from main.c

// TA1 (P1.6) = LED intensity PWM
#define LED_GATE_POUT   P4OUT
#define LED_RED         0x08                            // red LED gate, PORT4.3
#define LED_GREEN       0x10                            // green LED gate, PORT4.4
#define LED_BLUE        0x20                            // blue LED gate, PORT4.5

// turn LED(s) on(off)
void SetLED (int color, int state) __monitor {
    if (state) {                                        // turn on LED(s)
        switch (color) {
            case RED:
                LED_GATE_POUT |= LED_RED;               // red LED on
                flags |= F_RED;                         // update flag
                sampleCnt = 0;                          // reset LED sample count
                break;
            case GREEN:
                LED_GATE_POUT |= LED_GREEN;             // green LED on
                flags |= F_GREEN;                       // update flag
                sampleCnt = 0;                          // reset LED sample count
                break;
            case BLUE:
                LED_GATE_POUT |= LED_BLUE;              // blue LED on
                flags |= F_BLUE;                        // update flag
                sampleCnt = 0;                          // reset LED sample count
                break;
            case ALL:
                LED_GATE_POUT |= (LED_RED+LED_GREEN+LED_BLUE);          // all LEDs on
                flags |= (F_RED+F_GREEN+F_BLUE);        // update flags
                sampleCnt = 0;                          // reset LED sample count
                break;
        }
    } else {                                            // turn off LED(s)
        switch (color) {
            case RED:
                LED_GATE_POUT &= ~LED_RED;              // red LED off
                flags &= ~F_RED;                        // update flag
                sampleCnt = 0;                          // reset LED sample count
                break;
            case GREEN:
                LED_GATE_POUT &= ~LED_GREEN;            // green LED off
                flags &= ~F_GREEN;                      // update flag
                sampleCnt = 0;                          // reset LED sample count
                break;
            case BLUE:
                LED_GATE_POUT &= ~LED_BLUE;             // blue LED off
                flags &= ~F_BLUE;                       // update flag
                sampleCnt = 0;                          // reset LED sample count
                break;
            case ALL:
                LED_GATE_POUT &= ~(LED_RED+LED_GREEN+LED_BLUE);         // all LEDs off
                flags &= ~(F_RED+F_GREEN+F_BLUE);       // update flags
                sampleCnt = 0;                          // reset LED sample count
                break;
        }
    }
}


// set state for all LEDs
void SetAllLEDs (int leds_on, int leds_off) __monitor {
    int     mask_off = 0xff,
            mask_on = 0;
    ulong   mask_flags = 0;

    if (leds_off & RED) mask_off &= ~LED_RED;                   // create mask for turning LEDs off
    if (leds_off & GREEN) mask_off &= ~LED_GREEN;
    if (leds_off & BLUE) mask_off &= ~LED_BLUE;
    if (leds_on & RED) {
        mask_on |= LED_RED;                                     // create mask for turning LEDs on
        mask_flags |= F_RED;
    }
    if (leds_on & GREEN) {
        mask_on |= LED_GREEN;
        mask_flags |= F_GREEN;
    }
    if (leds_on & BLUE) {
        mask_on |= LED_BLUE;
        mask_flags |= F_BLUE;
    }
    LED_GATE_POUT &= mask_off;                                  // leds OFF
    LED_GATE_POUT |= mask_on;                                   // leds ON
    flags &= ~(F_RED+F_GREEN+F_BLUE);                           // clear all LED flags
    flags |= mask_flags;                                        // update LED flags
    sampleCnt = 0;                                              // reset LED sample count

}


// set LED intensity (PWM) ouput
bool Set_LEDintensity (float percent) __monitor {
    if ((percent >= 0)&&(percent <= 100)) {                     // validate
        TACCR1 = (uint)((TA_INTERVAL-1) * (percent / 100.0));   // update PWM counter register
        return true;
    } else {                                                    // error, must be between 0 and 100%
        return false;
    }
}


// read LED intensity
float Get_LEDintensity (void) {
    return ((float)TACCR1 / (float)TA_INTERVAL) * 100.0;
}


// set front-panel LED on/off
void SetFpLED (int state) {
    if (state) {
        FPLED_POUT |= FPLED;            // turn on LED
    } else {
        FPLED_POUT &= ~FPLED;           // turn off LED
    }
}
