#include "led.h"
#include "temperature.h"
#include <math.h>

// *********************************************************************
// calculate thermistor temperature given it's resistance
// and calibration coefficients
// *********************************************************************
float CalThermTemp(float res, float* cal) {
    float t,r;

    if (res < 1.0) {                                // validate resistance
        t = 0.0;                                    // invalid resistance
    } else {                                        // calculate temperature
        r = log(res);                               // natural log of resistance
        t = ((cal[3]*r + cal[2])*r + cal[1])*r + cal[0];
        if (t != 0.0) {                             // prevent div by 0
            t = 1.0 / t;
        }
    }
    return t - 273.15;                              // convert to C
}

// *********************************************************************
// calculate thermistor temperature given it's resistance and it's
// reference temperature, resistance, and sensitivity
// *********************************************************************
float ThermTemp (float res, float inv_ref_temp, float inv_ref_res, float inv_sens) {
    float t;

    t = log(res * inv_ref_res) * inv_sens + inv_ref_temp;
    if (t != 0.0) {                                 // prevent div by 0
        t = 1.0 / t;
    }
    t = t - 273.15;
    return t;
}

// *********************************************************************
// init Shibaura temperature coefficients
// *********************************************************************
void InitTaCoeff (float tc[]) {
	//6005
	tc[0] = +9.94659927e-04;
    tc[1] = +2.64668146e-04;
    tc[2] = 0.0;
    tc[3] = +1.34665959e-07;
}

// *********************************************************************
// init Humirel temperature coefficients
// *********************************************************************
void InitTuCoeff (float tc[]) {
	//6005
	tc[0] = +9.67751364e-04;
    tc[1] = +2.50963484e-04;
    tc[2] = 0.0;
    tc[3] = +1.02308736e-07;
}

// *********************************************************************
// init pressure temperature coefficients
// *********************************************************************
void InitTpCoeff (ThermChar* thc) {
    thc->inv_ref_temp = 1.0 / 298.15;   // 25 + 273.15 = deg K
    thc->inv_ref_res = 1.0 / 5000.0;    // 5 Kohm at 25 deg C
    thc->inv_sens = 1.0 / 3375.0;       // thermistor sensitivity
}

