#include "valve.h"
#include "global.h"

#define VALVE_16_POUT   P3OUT                       // valves #1..#6
#define VALVE_16_PIN    P3IN
#define VALVE_79_POUT   P2OUT                       // valves #7..#9
#define VALVE_79_PIN    P2IN
#define VALVE_10_POUT   P1OUT                       // valve #10
#define VALVE_10_PIN    P1IN
#define VALVE_16_MASK   0x30                        // bits NOT used by valves #1..#6
#define VALVE_79_MASK   0x1F                        // bits NOT used by valves #7..#9
#define VALVE_10_MASK   0x7F                        // bits NOT used by valve #10
#define VALVE_1         0x40                        // valve 1, PORT3.6 (ref 1)
#define VALVE_2         0x80                        // valve 2, PORT3.7
#define VALVE_3         0x08                        // valve 3, PORT3.3
#define VALVE_4         0x04                        // valve 4, PORT3.2
#define VALVE_5         0x02                        // valve 5, PORT3.1
#define VALVE_6         0x01                        // valve 6, PORT3.0
#define VALVE_7         0x80                        // valve 7, PORT2.7
#define VALVE_8         0x40                        // valve 8, PORT2.6
#define VALVE_9         0x20                        // valve 9, PORT2.5
#define VALVE_10        0x80                        // valve 10, PORT1.7 (ref 2)

// open the specified valve and close all others (0=all off)
void OpenValve (int valve) {
    switch (valve) {
        case 1:                                     // open valve #2 & reference #2, all others closed
            VALVE_79_POUT = VALVE_79_PIN & VALVE_79_MASK;                       // close valves #7..#9
            VALVE_10_POUT = (VALVE_10_PIN & VALVE_10_MASK) | VALVE_10;          // open valve #10 (ref 2)
            VALVE_16_POUT = (VALVE_16_PIN & VALVE_16_MASK) | VALVE_2;           // open valve #2, close #1 & #3..#6
            break;
        case 2:                                     // open valve #3 & reference #1, all others closed
            VALVE_79_POUT = VALVE_79_PIN & VALVE_79_MASK;                       // close valves #7..#9
            VALVE_10_POUT = (VALVE_10_PIN & VALVE_10_MASK);                     // close valve #10 (ref 2)
            VALVE_16_POUT = (VALVE_16_PIN & VALVE_16_MASK) | VALVE_1 | VALVE_3;         // open valves #1 (ref 1) & #3
            break;
        case 3:                                     // open valve #4 & reference #2, all others closed
            VALVE_79_POUT = VALVE_79_PIN & VALVE_79_MASK;                       // close valves #7..#9
            VALVE_10_POUT = (VALVE_10_PIN & VALVE_10_MASK) | VALVE_10;          // open valve #10 (ref 2)
            VALVE_16_POUT = (VALVE_16_PIN & VALVE_16_MASK) | VALVE_4;           // open valve #4
            break;
        case 4:                                     // open valve #5 & reference #1, all others closed
            VALVE_79_POUT = VALVE_79_PIN & VALVE_79_MASK;                       // close valves #7..#9
            VALVE_10_POUT = (VALVE_10_PIN & VALVE_10_MASK);                     // close valve #10 (ref 2)
            VALVE_16_POUT = (VALVE_16_PIN & VALVE_16_MASK) | VALVE_1 | VALVE_5;         // open valves #1 (ref 1) & #5
            break;
        case 5:                                     // open valve #6 & reference #2, all others closed
            VALVE_79_POUT = VALVE_79_PIN & VALVE_79_MASK;                       // close valves #7..#9
            VALVE_10_POUT = (VALVE_10_PIN & VALVE_10_MASK) | VALVE_10;          // open valve #10 (ref 2)
            VALVE_16_POUT = (VALVE_16_PIN & VALVE_16_MASK) | VALVE_6;           // open valve #6
            break;
        case 6:                                     // open valve #7 & reference #1, all others closed
            VALVE_10_POUT = (VALVE_10_PIN & VALVE_10_MASK);                     // close valve #10 (ref 2)
            VALVE_16_POUT = (VALVE_16_PIN & VALVE_16_MASK) | VALVE_1 ;          // open valve #1 (ref 1)
            VALVE_79_POUT = (VALVE_79_PIN & VALVE_79_MASK) | VALVE_7;           // open valve #7, close valves #8 & #9
            break;
        case 7:                                     // open valve #8 & reference #2, all others closed
            VALVE_16_POUT = VALVE_16_PIN & VALVE_16_MASK;                       // close valves #1..#6
            VALVE_10_POUT = (VALVE_10_PIN & VALVE_10_MASK) | VALVE_10;          // open valve #10 (ref 2)
            VALVE_79_POUT = (VALVE_79_PIN & VALVE_79_MASK) | VALVE_8;           // open valve #8
            break;
        case 8:                                     // open valve #9 & reference #1, all others closed
            VALVE_10_POUT = (VALVE_10_PIN & VALVE_10_MASK);                     // close valve #10 (ref 2)
            VALVE_16_POUT = (VALVE_16_PIN & VALVE_16_MASK) | VALVE_1;           // close valves #1..#6
            VALVE_79_POUT = (VALVE_79_PIN & VALVE_79_MASK) | VALVE_9;           // open valves #1 (ref 1) & #9
            break;
        default:                                    // all valves closed
            VALVE_10_POUT = VALVE_10_PIN & VALVE_10_MASK;                     // close valve #10
            VALVE_16_POUT = VALVE_16_PIN & VALVE_16_MASK;                     // close valves #1..#6
            VALVE_79_POUT = VALVE_79_PIN & VALVE_79_MASK;                     // close valves #7..#9
            break;
    }
}
